@@ -5,27 +5,13 @@
  * Date        : 2009-01-24
  * Description : Sane interface for KDE
  *
- * Copyright (C) 2007-2010 by Kare Sars <kare dot sars at iki dot fi>
- * Copyright (C) 2009 by Matthias Nagl <matthias at nagl dot info>
- * Copyright (C) 2009 by Grzegorz Kurtyka <grzegorz dot kurtyka at gmail dot com>
- * Copyright (C) 2007-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2014 by Gregor Mitsch: port to KDE5 frameworks
+ * SPDX-FileCopyrightText: 2007-2010 Kare Sars <kare dot sars at iki dot fi>
+ * SPDX-FileCopyrightText: 2009 Matthias Nagl <matthias at nagl dot info>
+ * SPDX-FileCopyrightText: 2009 Grzegorz Kurtyka <grzegorz dot kurtyka at gmail dot com>
+ * SPDX-FileCopyrightText: 2007-2008 Gilles Caulier <caulier dot gilles at gmail dot com>
+ * SPDX-FileCopyrightText: 2014 Gregor Mitsch : port to KDE5 frameworks
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) version 3, or any
- * later version accepted by the membership of KDE e.V. (or its
- * successor approved by the membership of KDE e.V.), which shall
- * act as a proxy defined in Section 6 of version 3 of the license.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
  *
  * ============================================================ */
 
@@ -40,8 +26,8 @@
 #include <QSplitter>
 #include <QMutex>
 #include <QPointer>
-#include <QDebug>
 #include <QIcon>
+#include <QShortcut>
 
 #include <kpassworddialog.h>
 #ifdef HAVE_KF5WALLET
@@ -59,12 +45,14 @@
 #include "ksanedevicedialog.h"
 #include "labeledgamma.h"
 
+#include <ksane_debug.h>
+
 namespace KSaneIface
 {
 static int     s_objectCount = 0;
-static QMutex  s_objectMutex;
+Q_GLOBAL_STATIC(QMutex, s_objectMutex)
 
-static const QString InvetColorsOption = QStringLiteral("KSane::InvertColors");
+static const QLatin1String InvetColorsOption("KSane::InvertColors");
 
 KSaneWidget::KSaneWidget(QWidget *parent)
     : QWidget(parent), d(new KSaneWidgetPrivate(this))
@@ -72,23 +60,23 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     SANE_Int    version;
     SANE_Status status;
 
-    s_objectMutex.lock();
+    s_objectMutex->lock();
     s_objectCount++;
 
     if (s_objectCount == 1) {
         // only call sane init for the first instance
         status = sane_init(&version, &KSaneAuth::authorization);
         if (status != SANE_STATUS_GOOD) {
-            qDebug() << "libksane: sane_init() failed("
+            qCDebug(KSANE_LOG) << "libksane: sane_init() failed("
                      << sane_strstatus(status) << ")";
         } else {
-            //qDebug() << "Sane Version = "
+            //qCDebug(KSANE_LOG) << "Sane Version = "
             //         << SANE_VERSION_MAJOR(version) << "."
             //         << SANE_VERSION_MINORparent(version) << "."
             //         << SANE_VERSION_BUILD(version);
         }
     }
-    s_objectMutex.unlock();
+    s_objectMutex->unlock();
 
     // read the device list to get a list of vendor and model info
     d->m_findDevThread->start();
@@ -159,15 +147,21 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     d->m_clearSelBtn->setToolTip(i18n("Clear Selections"));
     connect(d->m_clearSelBtn, SIGNAL(clicked()), d->m_previewViewer, SLOT(clearSelections()));
 
+    QShortcut *prevShortcut = new QShortcut(QKeySequence(QStringLiteral("Ctrl+P")), this);
+    connect(prevShortcut,   SIGNAL(activated()), d, SLOT(startPreviewScan()));
+
+    QShortcut *scanShortcut = new QShortcut(QKeySequence(QStringLiteral("Ctrl+S")), this);
+    connect(scanShortcut,   SIGNAL(activated()), d, SLOT(startFinalScan()));
+
     d->m_prevBtn = new QPushButton(this);
     d->m_prevBtn->setIcon(QIcon::fromTheme(QStringLiteral("document-import")));
-    d->m_prevBtn->setToolTip(i18n("Scan Preview Image"));
+    d->m_prevBtn->setToolTip(i18n("Scan Preview Image (%1)", prevShortcut->key().toString()));
     d->m_prevBtn->setText(i18nc("Preview button text", "Preview"));
     connect(d->m_prevBtn,   SIGNAL(clicked()), d, SLOT(startPreviewScan()));
 
     d->m_scanBtn = new QPushButton(this);
     d->m_scanBtn->setIcon(QIcon::fromTheme(QStringLiteral("document-save")));
-    d->m_scanBtn->setToolTip(i18n("Scan Final Image"));
+    d->m_scanBtn->setToolTip(i18n("Scan Final Image (%1)", scanShortcut->key().toString()));
     d->m_scanBtn->setText(i18nc("Final scan button text", "Scan"));
     d->m_scanBtn->setFocus(Qt::OtherFocusReason);
     connect(d->m_scanBtn,   SIGNAL(clicked()), d, SLOT(startFinalScan()));
@@ -241,11 +235,11 @@ KSaneWidget::KSaneWidget(QWidget *parent)
 KSaneWidget::~KSaneWidget()
 {
     while (!closeDevice()) {
-        usleep(1000);
+        usleep(100);
     }
     // wait for any thread to exit
 
-    s_objectMutex.lock();
+    s_objectMutex->lock();
     s_objectCount--;
     if (s_objectCount <= 0) {
         // only delete the find-devices and authorization singletons and call sane_exit
@@ -254,28 +248,43 @@ KSaneWidget::~KSaneWidget()
         delete d->m_auth;
         sane_exit();
     }
-    s_objectMutex.unlock();
+    s_objectMutex->unlock();
     delete d;
 }
 
+QString KSaneWidget::deviceName() const
+{
+    return d->m_devName;
+}
+
+QString KSaneWidget::deviceVendor() const
+{
+    return d->m_vendor;
+}
+
+QString KSaneWidget::deviceModel() const
+{
+    return d->m_model;
+}
+
 QString KSaneWidget::vendor() const
 {
     d->m_findDevThread->wait();
     d->devListUpdated(); // this is just a wrapped if (m_vendor.isEmpty()) statement if the vendor is known
     // devListUpdated here is to ensure that we do not come in between finished and the devListUpdated slot
-
     return d->m_vendor;
 }
+
 QString KSaneWidget::make() const
 {
     return vendor();
 }
+
 QString KSaneWidget::model() const
 {
     d->m_findDevThread->wait();
     d->devListUpdated(); // this is just a wrapped if (m_vendor.isEmpty()) statement if the vendor is known
     // devListUpdated here is to ensure that we do not come in between finished and the devListUpdated slot
-
     return d->m_model;
 }
 
@@ -298,14 +307,7 @@ QString KSaneWidget::selectDevice(QWidget *parent)
 
 void KSaneWidget::initGetDeviceList() const
 {
-    // update the device list if needed to get the vendor and model info
-    if (d->m_findDevThread->devicesList().size() == 0) {
-        //qDebug() << "initGetDeviceList() starting thread...";
-        d->m_findDevThread->start();
-    } else {
-        //qDebug() << "initGetDeviceList() have existing data...";
-        d->signalDevListUpdate();
-    }
+    d->m_findDevThread->start();
 }
 
 bool KSaneWidget::openDevice(const QString &deviceName)
@@ -395,7 +397,7 @@ bool KSaneWidget::openDevice(const QString &deviceName)
     }
 
     if (status != SANE_STATUS_GOOD) {
-        qDebug() << "sane_open(\"" << deviceName << "\", &handle) failed! status = " << sane_strstatus(status);
+        qCDebug(KSANE_LOG) << "sane_open(\"" << deviceName << "\", &handle) failed! status = " << sane_strstatus(status);
         d->m_auth->clearDeviceAuth(d->m_devName);
         d->m_devName.clear();
         return false;
@@ -461,12 +463,12 @@ bool KSaneWidget::openDevice(const QString &deviceName)
 
     // do the connections of the option parameters
     for (i = 0; i < d->m_optList.size(); ++i) {
-        //qDebug() << d->m_optList.at(i)->name();
+        //qCDebug(KSANE_LOG) << d->m_optList.at(i)->name();
         connect(d->m_optList.at(i), SIGNAL(optsNeedReload()), d, SLOT(optReload()));
         connect(d->m_optList.at(i), SIGNAL(valsNeedReload()), d, SLOT(scheduleValReload()));
 
         if (d->m_optList.at(i)->needsPolling()) {
-            //qDebug() << d->m_optList.at(i)->name() << " needs polling";
+            //qCDebug(KSANE_LOG) << d->m_optList.at(i)->name() << " needs polling";
             d->m_pollList.append(d->m_optList.at(i));
             KSaneOptCheckBox *buttonOption = qobject_cast<KSaneOptCheckBox *>(d->m_optList.at(i));
             if (buttonOption) {
@@ -626,7 +628,7 @@ QImage KSaneWidget::toQImageSilent(const QByteArray &data,
     }
     case FormatNone:
     default:
-        qDebug() << "Unsupported conversion";
+        qCDebug(KSANE_LOG) << "Unsupported conversion";
         break;
     }
     float dpm = dpi * (1000.0 / 25.4);
@@ -655,7 +657,7 @@ void KSaneWidget::scanFinal()
         d->startFinalScan();
     } else {
         // if the button frame is disabled, there is no open device to scan from
-        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+        Q_EMIT scanDone(KSaneWidget::ErrorGeneral, QString());
     }
 }
 
@@ -665,7 +667,7 @@ void KSaneWidget::startPreviewScan()
         d->startPreviewScan();
     } else {
         // if the button frame is disabled, there is no open device to scan from
-        emit scanDone(KSaneWidget::ErrorGeneral, QStringLiteral(""));
+        Q_EMIT scanDone(KSaneWidget::ErrorGeneral, QString());
     }
 }
 
@@ -678,6 +680,7 @@ void KSaneWidget::scanCancel()
     if (d->m_previewThread->isRunning()) {
         d->m_previewThread->cancelScan();
     }
+    d->m_cancelMultiScan = true;
 }
 
 void KSaneWidget::setPreviewResolution(float dpi)
@@ -891,7 +894,7 @@ float KSaneWidget::scanAreaWidth()
             d->m_optBrX->getMaxValue(result);
             float dpi = currentDPI();
             if (dpi < 1) {
-                qDebug() << "Broken DPI value";
+                qCDebug(KSANE_LOG) << "Broken DPI value";
                 dpi = 1.0;
             }
             result = result / dpi / 25.4;
@@ -910,7 +913,7 @@ float KSaneWidget::scanAreaHeight()
             d->m_optBrY->getMaxValue(result);
             float dpi = currentDPI();
             if (dpi < 1) {
-                qDebug() << "Broken DPI value";
+                qCDebug(KSANE_LOG) << "Broken DPI value";
                 dpi = 1.0;
             }
             result = result / dpi / 25.4;
